from ftw.theming.interfaces import IThemingLayer


def set_ftw_theming_header(object, event):
    """Set an HTTP_X_FTW_THEMING header in the request when ftw.theming is enabled.

    For enabling ftw.theming, the ``ftw.theming:default`` profile must
    be installed, enabling the required request layer.
    """

    request = event.request
    if not IThemingLayer.providedBy(request):
        return
    request.environ['HTTP_X_FTW_THEMING'] = True



from ftw.builder import Builder
from ftw.builder import create
from ftw.theming.profileinfo import ProfileInfo
from ftw.theming.tests import FunctionalTestCase
from Products.CMFCore.utils import getToolByName


class TestProfileInfo(FunctionalTestCase):

    def setUp(self):
        super(TestProfileInfo, self).setUp()
        self.package_builder = (Builder('python package')
                                .at_path(self.layer['temp_directory'])
                                .named('the.package')
                                .with_profile(Builder('genericsetup profile')
                                              .with_fs_version('1000')))
        self.portal_setup = getToolByName(self.portal, 'portal_setup')
        self.quickinstaller = getToolByName(self.portal, 'portal_quickinstaller')

    def test_is_profile_installed(self):
        with create(self.package_builder).zcml_loaded(self.layer['configurationContext']):
            info = ProfileInfo(self.portal)
            self.assertFalse(info.is_profile_installed('profile-the.package:default'))
            self.assertFalse(info.is_profile_installed('the.package:default'))

            info = ProfileInfo(self.portal)
            self.portal_setup.runAllImportStepsFromProfile('profile-the.package:default')
            self.assertTrue(info.is_profile_installed('profile-the.package:default'))
            self.assertTrue(info.is_profile_installed('the.package:default'))

            info = ProfileInfo(self.portal)
            self.quickinstaller.uninstallProducts(['the.package'])
            self.assertFalse(info.is_profile_installed('profile-the.package:default'))
            self.assertFalse(info.is_profile_installed('the.package:default'))



from ftw.theming.interfaces import ISCSSFileResource
from ftw.theming.interfaces import ISCSSRegistry
from Products.Five import BrowserView
from zope.component import getUtility


class ThemingResources(BrowserView):

    def resource_items(self):
        for index, resource in enumerate(self.resources(), 1):
            available = resource.available(self.context, self.request)
            cssclasses = 'theming-resource '
            cssclasses += available and 'available' or 'not-available'

            infos = {
                'available': available,
                'cssclasses': cssclasses,
                'index': index,
                'name': resource.name,
                'slot': resource.slot,
                }

            if ISCSSFileResource.providedBy(resource):
                infos.update({
                        'path': resource.path,
                        'package': resource.package,
                        'profile': resource.profile,
                        'for_name': resource.for_.__name__,
                        'for_path': resource.for_.__identifier__,
                        'layer_name': resource.layer.__name__,
                        'layer_path': resource.layer.__identifier__,
                        })

            yield infos

    def resources(self):
        registry = getUtility(ISCSSRegistry)
        return registry.get_resources(self.context, self.request,
                                      include_unavailable=True)



from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.theme.interfaces import IDefaultPloneLayer
from zope.interface import Attribute
from zope.interface import Interface


SLOTS = ('top',
         'variables',
         'mixins',
         'ftw.theming',
         'addon',
         'theme',
         'policy',
         'bottom')


class IThemingLayer(IDefaultPloneLayer):
    """A browser layer provided by the request when the ftw.theming
    Generic Setup profile is installed.
    """


class ISCSSCompiler(Interface):
    """Converts the SCSS to CSS.
    """

    def __init__(context, request):
        """A multi-adapter, adapting a context and a request.
        """

    def compile(debug=False):
        """Compile the scss files looked up from the SCSS registry
        into one CSS file.

        :param debug: Enabling debug mode adds sass debug infos for
          browser plugins.
        :type debug: bool (default: ``False``)
        :returns: The compiled CSS.
        :rtype: utf-8 encoded bytestring
        """


class ISCSSRegistry(Interface):
    """The scss registry holds all registered ISCSSFileResource objects.
    It decides which resources are used for compiling to CSS.
    """

    def add_resource(resource):
        """Add a scss resource to the registry.

        :param resource: A scss resource object.
        :type resource: :py:class:`ftw.theming.interfaces.ISCSSFileResource`
        """

    def get_resources(context, request, profileinfo=None,
                      include_unavailable=False):
        """Return the resources for a context and a request.
        The resources are filtered and sorted by slots.

        :param context: A acquisition wrapped context object.
        :type context: object
        :param request: The request object.
        :param request: object
        :param profileinfo: A profileinfo object for checking whether
          the profile is installed.
        :type profile: :py:class:`ftw.theming.profileinfo.ProfileInfo`
        :param include_unavailable: Disable filtering unavailable resources.
        :type include_unavailable: bool (default: False)
        :returns: A list of scss resource objects.
        :rtype: list of :py:class:`ftw.theming.interfaces.ISCSSFileResource`
        """

    def get_raw_dynamic_resources(context, request):
        """Returns all dynamic resources without ordering or filtering them.
        This is an efficient lookup for doing such as fast cache calculation.

        :param context: A acquisition wrapped context object.
        :type context: object
        :param request: The request object.
        :param request: object
        :returns: A list of scss resource objects.
        :rtype: list of :py:class:`ftw.theming.interfaces.IDynamicSCSSResource`
        """


class ISCSSResource(Interface):
    """An SCSS resource has a snippet of SCSS code and can tell whether the
    resources should be rendered an a specific context / request / setup.
    """

    name = Attribute('The name of the resource.')
    slot = Attribute('The slot where the resource belongs.'
                     ' This must be one of the list of slots.')
    before = Attribute('Move this resource before the other resource with that'
                       ' name within the same slot.')
    after = Attribute('Move this resource after the other resource with that'
                      ' name within the same slot.')

    def __init__(name, slot='addon', before=None, after=None, source=u''):
        """Initialize an scss resource.

        :param name: The name of the resource.
        :type name: string
        :param slot: The slot where the resource belongs. This must be one of
          the list of slots.
        :type slot: string
        :param before: Move this resource before the other resource with that
          name within the same slot.
        :type before: string (name of other resource)
        :param after: Move this resource after the other resource with that
          name within the same slot.
        :type after: string (name of other resource)
        :param source: The SCSS source.
        :type source: string
        """

    def available(context, request, profileinfo=None):
        """Check whether the resource is available for this context.

        Checks applied:

        - ``context`` must provide the ``for_``-interface of the resorce
        - ``request`` must provide the ``layer``-interface of the resorce
        - the profile must be installed.

        :param context: A acquisition wrapped context object.
        :type context: object
        :param request: The request object.
        :type request: object
        :param profileinfo: A profileinfo object for checking whether
          the profile is installed.
        :type profile: :py:class:`ftw.theming.profileinfo.ProfileInfo`
        :returns: ``True`` when the resource is available and should be
          included.
        :rtype: bool
        """

    def get_source(context, request):
        """Returns the raw SCSS of this resource as string.

        :param context: A acquisition wrapped context object.
        :type context: object
        :param request: The request object.
        :type request: object
        :returns: The raw SCSS
        :rtype: string
        """

    def get_cachekey(context, request):
        """Returns a string which changes when the content source
        changes.

        :param context: A acquisition wrapped context object.
        :type context: object
        :param request: The request object.
        :type request: object
        :returns: cache string
        :rtype: string
        """


class IDynamicSCSSResource(ISCSSResource):
    """A dynamic SCSS resource provides SCSS source which may change.
    In order to have the correct caching, a cachekey is necessary.

    Dynamic resources should either subclass the DynamicSCSSResource class or
    initialize it with at least a name, a source and a cache key.
    """

    def __init__(name, slot='addon', before=None, after=None, source=u''):
        """Initialize an scss resource.

        :param name: The name of the resource.
        :type name: string
        :param slot: The slot where the resource belongs. This must be one of
          the list of slots.
        :type slot: string
        :param before: Move this resource before the other resource with that
          name within the same slot.
        :type before: string (name of other resource)
        :param after: Move this resource after the other resource with that
          name within the same slot.
        :type after: string (name of other resource)
        :param source: The SCSS source.
        :type source: string
        :param cachekey: The cache key which must change when the source changes.
        :type source: string
        """


class ISCSSFileResource(ISCSSResource):
    """A scss resource represents a scss file for registering in the scss registry.
    It holds the relevant information for building the scss pipeline.

    Resources are identified by name. The name of the resource contains the
    package dottedname and the relative path to the file,
    e.g. "my.package.browser/resources/style.scss"
    """

    def __init__(package, relative_path, slot='addon',
                 profile=None, for_=INavigationRoot, layer=Interface,
                 before=None, after=None):
        """Initialize an scss resource.

        :param package: The name of the python package where the resource is
          registered.
        :type package: string
        :param relative_path: The path to the scss file relative to the
          package directory.
        :type relative_path: string
        :param slot: The slot where the resource belongs. This must be one of
          the list of slots.
        :type slot: string
        :param profile: The name of the Generic Setup profile,
          e.g. "profile-my.package:default".
        :type profile: string
        :param for_: The context interface for which this resource is
          registered. This is usually the Plone site, but may also be
          another navigation root.
        :type for_: :py:class:`zope.interface.Interface`
        :param layer: The request layer interface for which this resource
          is registered.
        :type layer: :py:class:`zope.interface.Interface`
        :param before: Move this resource before the other resource with that
          name within the same slot.
        :type before: string (name of other resource)
        :param after: Move this resource after the other resource with that
          name within the same slot.
        :type after: string (name of other resource)
        """


class ISCSSResourceFactory(Interface):
    """An SCSS resource factory creates and returnes an SCSSResource when called.
    """

    def __call__(context, request):
        """Accepts any context and request and returns an SCSSResource object.
        """


class ICSSCaching(Interface):
    """Marker interface for enabling plone.app.caching on the theming resource.
    """



from collections import OrderedDict
from ftw.theming.interfaces import ISCSSCompiler
from ftw.theming.utils import find_object_in_stack
from operator import methodcaller
from Products.Five import BrowserView
from scss.calculator import Calculator
from scss.types import Null
from zope.component import getMultiAdapter


class ThemingVariablesView(BrowserView):

    def packages(self):
        packages = OrderedDict()
        compiler = getMultiAdapter((self.context, self.request), ISCSSCompiler)

        @compiler.namespace.declare
        def register_theming_variables(*varnames):
            calculator = find_object_in_stack('self', Calculator)
            variables = calculator.namespace.variables
            package = (variables.get('$current-package').render()
                       .strip('"').strip("'"))
            filename = (variables.get('$current-filename').render()
                        .strip('"').strip("'"))
            varnames = map(methodcaller('render'), varnames)

            if package not in packages:
                packages[package] = []

            for name in varnames:
                name = u'${0}'.format(name)
                value = variables.get(name, None)
                vartype = value.__class__.__name__
                value = value and value.render or '???'
                packages[package].append({'filename': filename,
                                           'name': name,
                                           'value': value,
                                           'type': vartype})
            return Null()

        compiler.compile()
        return ({'name': key, 'variables': values}
                for key, values in packages.items())



from ftw.theming.extensions import ThemingExtensions
from ftw.theming.interfaces import ISCSSCompiler
from ftw.theming.interfaces import ISCSSFileResource
from ftw.theming.interfaces import ISCSSRegistry
from pathlib import Path
from scss.compiler import Compiler
from scss.namespace import Namespace
from scss.source import SourceFile
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implements
from zope.interface import Interface
import hashlib
import os.path


class SCSSCompiler(object):
    implements(ISCSSCompiler)
    adapts(Interface, Interface)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.namespace = Namespace()

    def compile(self, debug=False):
        return self._compile(self._get_scss_files(), debug=debug)

    def compile_scss_string(self, source, debug=False):
        source_file = SourceFile.from_string(source)
        return self._compile((source_file,), debug=debug)

    def get_cachekey(self, dynamic_resources_only=False, **kwargs):
        registry = getUtility(ISCSSRegistry)

        if dynamic_resources_only:
            # Usually used in production mode: omit file resources and skip
            # ordering in order to speed up cache generation.
            resources = registry.get_raw_dynamic_resources(self.context, self.request)

        else:
            # Usually used in development mode: include file's modified date
            # in order to be up to date when developing.
            resources = registry.get_resources(self.context, self.request)

        result = hashlib.md5()
        map(result.update, (resource.get_cachekey(self.context, self.request)
                            for resource in resources))
        return result.hexdigest()

    def _get_scss_files(self):
        def make_source_file(resource):
            package = resource.name.split(':')[0]
            relpath = resource.name.split(':')[-1]
            filename = relpath.split('/')[-1]
            source = u'\n'.join((
                    u'$current-package: "{0}";'.format(package),
                    u'$current-relpath: "{0}";'.format(relpath),
                    u'$current-filename: "{0}";'.format(filename),
                    resource.get_source(self.context, self.request)))
            if ISCSSFileResource.providedBy(resource):
                source_file = SourceFile.from_string(
                    source, Path(str(resource.path)))
                source_file.origin = Path(str(resource.path))
                return source_file
            else:
                path = 'dynamic:{0}'.format(resource.name)
                return SourceFile.from_string(source, path)

        registry = getUtility(ISCSSRegistry)
        resources = registry.get_resources(self.context, self.request)
        return map(make_source_file, resources)

    def _compile(self, files, debug=False):
        compiler = Compiler(
            search_path=self._get_search_path_for_files(files),
            output_style=debug and 'expanded' or 'compressed',
            generate_source_map=True,
            namespace=self.namespace)
        compiler.extensions.append(ThemingExtensions())
        css = compiler.compile_sources(*files)
        return css.encode('utf-8')

    def _get_search_path_for_files(self, files):
        directories = []
        for source_file in files:
            if not source_file.path or not source_file.path.startswith('/'):
                continue
            path = os.path.dirname(source_file.path)
            if path not in directories:
                directories.append(path)
        return map(Path, directories)



from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.tests import FunctionalTestCase
from zope.component import getUtility


class TestVariables(FunctionalTestCase):

    def test_portal_url_variable_resource(self):
        registry = getUtility(ISCSSRegistry)
        resources = dict((res.name, res)
                         for res in registry.get_resources(self.portal, self.request))
        self.assertIn('ftw.theming:portal_url', resources.keys())

        resource = resources['ftw.theming:portal_url']
        self.assertEquals('$portal-url: "http://nohost/plone";\n'
                          '@include declare-variables(portal-url);',
                          resource.get_source(self.portal, self.request).strip())

        self.assertEquals('variables', resource.slot)

    def test_plone_image_sizes(self):
        registry = getUtility(ISCSSRegistry)
        resources = dict((res.name, res)
                         for res in registry.get_resources(self.portal, self.request))
        self.assertIn('ftw.theming:image_sizes', resources.keys())

        resource = resources['ftw.theming:image_sizes']
        self.assertEquals('variables', resource.slot)

        self.maxDiff = None

        self.assertEqual((
            '$scale-mini-width: 200px;'
            '$scale-mini-height: 200px;'
            '@include declare-variables(scale-width-mini, scale-height-mini);'
            '$scale-thumb-width: 128px;'
            '$scale-thumb-height: 128px;'
            '@include declare-variables(scale-width-thumb, scale-height-thumb);'
            '$scale-large-width: 768px;'
            '$scale-large-height: 768px;'
            '@include declare-variables(scale-width-large, scale-height-large);'
            '$scale-listing-width: 16px;'
            '$scale-listing-height: 16px;'
            '@include declare-variables(scale-width-listing, scale-height-listing);'
            '$scale-tile-width: 64px;'
            '$scale-tile-height: 64px;'
            '@include declare-variables(scale-width-tile, scale-height-tile);'
            '$scale-preview-width: 400px;'
            '$scale-preview-height: 400px;'
            '@include declare-variables(scale-width-preview, scale-height-preview);'
            '$scale-icon-width: 32px;'
            '$scale-icon-height: 32px;'
            '@include declare-variables(scale-width-icon, scale-height-icon);'
            ), resource.get_source(self.portal, self.request).strip())



from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.resource import SCSSResource
from ftw.theming.testing import META_ZCML
from ftw.theming.tests.stubs import CONTEXT
from ftw.theming.tests.stubs import ProfileInfoStub
from ftw.theming.tests.stubs import REQUEST
from operator import attrgetter
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from unittest2 import TestCase
from zope.component import getUtility
from zope.configuration.xmlconfig import ZopeXMLConfigurationError
from zope.interface import Interface
from zope.interface import provider
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
import ftw.theming.tests


class TestSCSSDirective(TestCase):
    layer = META_ZCML

    def test_register_resource(self):
        self.load_zcml('<theme:scss file="resources/foo.scss" />')
        self.assertDictContainsSubset(
            {'name': 'ftw.theming.tests:resources/foo.scss',
             'package': 'ftw.theming.tests',
             'relative_path': 'resources/foo.scss',
             'slot': 'addon',
             'profile': None,
             'for_': INavigationRoot,
             'layer': Interface,
             'before': None,
             'after': None},
            self.get_resource_vars())

    def test_slot(self):
        self.load_zcml('<theme:scss file="resources/foo.scss" slot="theme" />')
        self.assertDictContainsSubset({'slot': 'theme'},
                                      self.get_resource_vars())

    def test_valid_slot_required(self):
        with self.assertRaises(ZopeXMLConfigurationError) as cm:
            self.load_zcml('<theme:scss file="resources/foo.scss"'
                           ' slot="any" />')
        self.assertEquals('Invalid slot "any". Valid slots:'
                          ' (\'top\', \'variables\', \'mixins\', \'ftw.theming\','
                          ' \'addon\', \'theme\', \'policy\', \'bottom\')',
                          str(cm.exception.evalue))

    def test_profile(self):
        self.load_zcml('<theme:scss file="resources/foo.scss"'
                       ' profile="foo:default" />')
        self.assertDictContainsSubset({'profile': 'foo:default'},
                                      self.get_resource_vars())

    def test_for(self):
        self.load_zcml(
            '<theme:scss file="resources/foo.scss"'
            ' for="Products.CMFPlone.interfaces.siteroot.IPloneSiteRoot" />')
        self.assertDictContainsSubset({'for_': IPloneSiteRoot},
                                      self.get_resource_vars())

    def test_layer(self):
        self.load_zcml(
            '<theme:scss file="resources/foo.scss"'
            ' layer="zope.publisher.interfaces.browser.IDefaultBrowserLayer"'
            ' />')
        self.assertDictContainsSubset({'layer': IDefaultBrowserLayer},
                                      self.get_resource_vars())

    def test_before(self):
        self.load_zcml(
            '<theme:scss file="resources/foo.scss" before="bar:bar.scss" />')
        self.assertDictContainsSubset({'before': 'bar:bar.scss'},
                                      self.get_resource_vars())

    def test_after(self):
        self.load_zcml(
            '<theme:scss file="resources/foo.scss" after="bar:bar.scss" />')
        self.assertDictContainsSubset({'after': 'bar:bar.scss'},
                                      self.get_resource_vars())

    def test_complex_directive(self):
        self.load_zcml(
            '<theme:resources'
            '      slot="theme"'
            '      profile="foo:default"'
            '      for="Products.CMFPlone.interfaces.siteroot.IPloneSiteRoot"'
            '      layer="zope.publisher.interfaces.browser.IDefaultBrowserLayer">'
            '  <theme:scss'
            '        file="resources/foo.scss"'
            '        before="bar:bar.scss"'
            '        after="baz:baz.scss" />'
            '</theme:resources>')

        self.assertDictContainsSubset(
            {'name': 'ftw.theming.tests:resources/foo.scss',
             'package': 'ftw.theming.tests',
             'relative_path': 'resources/foo.scss',
             'slot': 'theme',
             'profile': 'foo:default',
             'for_': IPloneSiteRoot,
             'layer': IDefaultBrowserLayer,
             'before': 'bar:bar.scss',
             'after': 'baz:baz.scss'},
            self.get_resource_vars())

    def test_complex_directive_with_changed_slot(self):
        self.load_zcml(
            '<theme:resources slot="theme">'
            '  <theme:scss'
            '        file="resources/foo.scss"'
            '        slot="variables" />'
            '</theme:resources>')

        self.assertDictContainsSubset(
            {'name': 'ftw.theming.tests:resources/foo.scss',
             'package': 'ftw.theming.tests',
             'relative_path': 'resources/foo.scss',
             'slot': 'variables',
             'profile': None,
             'for_': INavigationRoot,
             'layer': Interface,
             'before': None,
             'after': None},
            self.get_resource_vars())

    def test_scss_factory(self):
        @provider(ISCSSResourceFactory)
        def resource_factory(context, request):
            return SCSSResource('foo', source='$color = red;')

        ftw.theming.tests.resource_factory = resource_factory
        self.load_zcml('<theme:scss_factory factory=".resource_factory" />')

        self.assertDictContainsSubset(
            {'name': 'foo',
             'slot': 'addon',
             'before': None,
             'after': None},
            self.get_resource_vars())

    def test_scss_factory_must_provide_interface(self):
        def bad_resource_factory(context, request):
            return None
        ftw.theming.tests.resource_factory = bad_resource_factory
        with self.assertRaises(ZopeXMLConfigurationError) as cm:
            self.load_zcml('<theme:scss_factory factory=".resource_factory" />')
        self.assertEquals('add_scss: factory must provide ISCSSResourceFactory',
                          str(cm.exception.evalue))

    def load_zcml(self, *lines):
        self.layer.load_zcml_string('\n'.join((
                    '<configure ',
                    '    xmlns:theme="http://namespaces.zope.org/ftw.theming"',
                    '    i18n_domain="my.package"',
                    '    package="ftw.theming.tests"''>',
                    ) + lines + (
                    '</configure>',
                    )))

    def get_resources(self, context=CONTEXT, request=REQUEST,
                      profileinfo=ProfileInfoStub(),
                      include_unavailable=True):
        registry = getUtility(ISCSSRegistry)
        return registry.get_resources(context, request,
                                      profileinfo=profileinfo,
                                      include_unavailable=include_unavailable)

    def get_resource(self, **kwargs):
        resource, = self.get_resources(**kwargs)
        return resource

    def get_resource_vars(self):
        return vars(self.get_resource())


def attrs(name, resources):
    return map(attrgetter(name), resources)



from ftw.theming.browser.theming_css import get_theming_css_url
from plone.app.layout.viewlets.common import ViewletBase


class ThemingCSS(ViewletBase):

    template = '<link rel="stylesheet" type="text/css" href="{href}" />'

    def render(self):
        return self.template.format(href=get_theming_css_url(self.context))



from ftw.theming.interfaces import ISCSSCompiler
from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.resource import DynamicSCSSResource
from ftw.theming.tests import FunctionalTestCase
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface.verify import verifyObject


class TestCompiler(FunctionalTestCase):

    def test_default_compiler_implements_interface(self):
        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        verifyObject(ISCSSCompiler, compiler)

    def test_compiles_to_css(self):
        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        result = compiler.compile()
        self.assertTrue(result, 'Compiled CSS should be empty.')
        self.assertEquals(str, type(result),
                          'Compiled CSS should be an utf-8 bytestring.')

    def test_compiling_with_debug_mode(self):
        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        self.assertTrue(compiler.compile(debug=True),
                        'Compiled CSS should be empty.')

    def test_compile_scss_string(self):
        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        result = compiler.compile_scss_string('$red: red;'
                                              'body { background-color: $red; }')
        self.assertEquals('body{background-color:red}', result.strip())

    def test_get_cachekey_returns_a_value(self):
        # The cachekey is based on the modified date of the resource,
        # which changes and therefore we just test that it is positive.
        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        self.assertTrue(compiler.get_cachekey())

    def test_get_cachkey_only_with_dynamic_resources(self):
        """In production we only respect cachekeys of dynamic resources in order
        to speed up cachekey calculation and because everything else should be
        static. We also do not want to change cache keys in production too often.

        Therefore the compiler's get_cachekey method has a dynamic_resources_only
        param.
        """
        resource = DynamicSCSSResource('foo', cachekey='foo')
        getUtility(ISCSSRegistry).add_resource(resource)

        compiler = getMultiAdapter((self.portal, self.request), ISCSSCompiler)
        initial_cachekey = compiler.get_cachekey(dynamic_resources_only=True)
        self.assertEqual(initial_cachekey,
                         compiler.get_cachekey(dynamic_resources_only=True))
        resource.cachekey = 'bar'
        self.assertNotEqual(initial_cachekey,
                            compiler.get_cachekey(dynamic_resources_only=True))



from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.registry import SCSSRegistry
from ftw.theming.resource import SCSSFileResource
from path import Path
from zope.component import provideUtility
from zope.component import queryUtility
from zope.configuration import fields
from zope.interface import Interface
from zope.schema import TextLine


class IResourcesDirective(Interface):

    slot = TextLine(
        title=u'The name of the slot where the resource should be'
        ' rendered in.',
        required=False)

    profile = TextLine(
        title=u'The name of the profile which needs to be installed'
        u' for this resource to be activated.',
        required=False)

    for_ = fields.GlobalInterface(
        title=u'The interface the context should provide.',
        required=False)

    layer = fields.GlobalInterface(
        title=u'The interface the request should provide.',
        required=False)


class IAddSCSSSubDirective(Interface):

    slot = TextLine(
        title=u'The name of the slot where the resource should be'
        ' rendered in.',
        required=False)

    file = fields.Path(
        title=u'Relative path to the .scss file.',
        required=True)

    before = TextLine(
        title=u'The name of the resource after which this resource should'
        ' be ordered.',
        description=u'The name consists of "package:relative file path"',
        required=False)

    after = TextLine(
        title=u'The name of the resource after which this resource should'
        ' be ordered.',
        description=u'The name consists of "package:relative file path"',
        required=False)


class IAddSCSSDirective(IResourcesDirective, IAddSCSSSubDirective):
    pass


def add_scss(context, **kwargs):
    """Register an .scss file.
    """

    registry = get_or_create_registry()

    relpath = Path(kwargs['file']).relpath(
        Path(context.package.__file__).parent)
    resource_args = {
        'package': context.package.__name__,
        'relative_path': relpath}

    for name in ('slot', 'profile', 'for_', 'layer', 'before', 'after'):
        if name in kwargs:
            resource_args[name] = kwargs[name]

    registry.add_resource(SCSSFileResource(**resource_args))


class Resources(object):

    def __init__(self, context, **kwargs):
        self.context = context
        self.kwargs = kwargs

    def scss(self, context, **kwargs):
        arguments = self.kwargs.copy()
        arguments.update(kwargs)
        add_scss(context, **arguments)


class IAddSCSSFactoryDirective(Interface):

    factory = fields.GlobalObject(
        title=u'Adapter factory',
        required=True)


def add_scss_factory(context, factory):
    """Register an SCSS resource factory.
    """

    if not ISCSSResourceFactory.providedBy(factory):
        raise ValueError('add_scss: factory must provide ISCSSResourceFactory')

    registry = get_or_create_registry()
    registry.add_resource(factory)


def get_or_create_registry():
    registry = queryUtility(ISCSSRegistry)
    if registry is None:
        registry = SCSSRegistry()
        provideUtility(registry)
    return registry



from ftw.testbrowser import browsing
from ftw.theming.tests import FunctionalTestCase
from plone.app.testing import SITE_OWNER_NAME


class TestControlpanel(FunctionalTestCase):

    @browsing
    def test_controlpanel_is_available(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='overview-controlpanel')
        browser.find('Theming Resources').click()
        self.assertEquals('http://nohost/plone/@@theming-resources',
                          browser.url)

    @browsing
    def test_navigate_from_resources_to_variables(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-resources')
        browser.find('Theming Variables').click()
        self.assertEquals('http://nohost/plone/@@theming-variables',
                          browser.url)

    @browsing
    def test_navigate_from_variables_to_resources(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-variables')
        browser.find('Theming Resources').click()
        self.assertEquals('http://nohost/plone/@@theming-resources',
                          browser.url)

    @browsing
    def test_navigate_from_resources_to_icons(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-resources')
        browser.find('Icons').click()
        self.assertEquals('http://nohost/plone/@@theming-icons',
                          browser.url)

    @browsing
    def test_theming_resources_lists_resources(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-resources')
        resources = browser.css('table.theming-resources').first.dicts()
        for resource in resources:
            del resource['']
            del resource['#']

        self.assertIn(
            {'For': 'INavigationRoot',
             'Layer': 'Interface',
             'Name': 'ftw.theming:resources/scss/default/controlpanel.scss',
             'Package': 'ftw.theming',
             'Profile': 'ftw.theming:default',
             'Slot': 'addon'},
            resources)

    @browsing
    def test_theming_variables_lists_variables(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-variables')
        variables = browser.css('table.theming-variables').first.dicts()

        self.assertIn({'File': 'variables.scss',
                       'Name': '$color-primary',
                       'Value': '#3B234A',
                       'Example': ''},
                      variables)

        self.assertIn({'File': 'portal_url',
                       'Name': '$portal-url',
                       'Value': '"http://nohost/plone"',
                       'Example': '"http://nohost/plone"'},
                      variables)

    @browsing
    def test_theming_icons_lists_portal_type_icons(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-icons')
        icons = browser.css('table.theming-portal-type-icons').first.dicts()
        self.assertIn({'Type': 'Folder',
                       'Name': 'folder'}, icons)

    @browsing
    def test_theming_icons_lists_mime_type_icons(self, browser):
        browser.login(SITE_OWNER_NAME).open(view='theming-icons')
        icons = browser.css('table.theming-mime-type-icons').first.dicts()
        self.assertIn({'Name': 'pdf', 'Original': '', 'Icon': ''}, icons)



from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.resource import DynamicSCSSResource
from plone.namedfile.interfaces import IAvailableSizes
from Products.CMFCore.utils import getToolByName
from zope.component import queryUtility
from zope.interface import provider
import hashlib


@provider(ISCSSResourceFactory)
def portal_url_variable_resource_factory(context, request):
    portal_url = getToolByName(context, 'portal_url')().decode('utf-8')
    cachekey = hashlib.md5(portal_url).hexdigest()
    return DynamicSCSSResource(
        'ftw.theming:portal_url', slot='variables',
        source=u'\n'.join((
               u'$portal-url: "{0}";'.format(portal_url),
               u'@include declare-variables(portal-url);')),
        cachekey=cachekey)


@provider(ISCSSResourceFactory)
def plone_image_sizes(context, request):
    sass_variables_template = (
        '$scale-{name}-width: {width}px;'
        '$scale-{name}-height: {height}px;'
        '@include declare-variables(scale-width-{name}, scale-height-{name});'
        )
    getAvailableSizes = queryUtility(IAvailableSizes)
    sizes = getAvailableSizes()

    def build_sass_map(size):
        width, height = size[1]
        name = size[0]
        return sass_variables_template.format(
            name=name, width=width, height=height)

    cachekey = hashlib.md5(str(sorted(sizes.values()))).hexdigest()
    sizes = map(build_sass_map, sizes.iteritems())

    return DynamicSCSSResource(
        'ftw.theming:image_sizes', slot='variables',
        source=u''.join(sizes),
        cachekey=cachekey)



from operator import attrgetter
from operator import methodcaller
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from Products.Five import BrowserView
from zope.component import getUtility
import re


class ThemingIcons(BrowserView):

    def get_portal_types(self):
        portal_types = getToolByName(self.context, 'portal_types')
        return map(methodcaller('getId'),
                   filter(IDynamicViewTypeInformation.providedBy,
                          portal_types.objectValues()))

    def get_mime_types(self):
        mimetypes_registry = getToolByName(self.context, 'mimetypes_registry')
        icon_paths = sorted(set(map(attrgetter('icon_path'),
                                    mimetypes_registry.mimetypes())))
        normalizer = getUtility(IIDNormalizer)

        def itemize(filename):
            name = normalizer.normalize(re.sub(r'\.png$', r'', filename))
            return {
                'filename': filename,
                'classes': 'mimetype-icon icon-mimetype-img-{}'.format(name),
                'normalized_name': name}

        return map(itemize, icon_paths)



from ftw.theming.interfaces import IDynamicSCSSResource
from ftw.theming.interfaces import ISCSSResource
from ftw.theming.resource import DynamicSCSSResource
from ftw.theming.resource import SCSSResource
from ftw.theming.tests.stubs import CONTEXT
from ftw.theming.tests.stubs import REQUEST
from unittest2 import TestCase
from zope.interface.verify import verifyObject


class TestSCSSResource(TestCase):

    def test_implements_interface(self):
        resource = SCSSResource('foo.scss')
        verifyObject(ISCSSResource, resource)

    def test_create_resource(self):
        resource = SCSSResource('foo.scss')
        self.assertTrue(resource)

    def test_requires_valid_slot(self):
        with self.assertRaises(ValueError):
            SCSSResource('foo.scss', slot='not existing slot')

    def test_available__always_returns_True(self):
        resource = SCSSResource('foo.scss')
        self.assertTrue(resource.available(CONTEXT, REQUEST))
        self.assertTrue(resource.available(None, None))

    def test_get_source(self):
        resource = SCSSResource('foo.scss', source=u'$foreground = black;')
        self.assertEquals(u'$foreground = black;',
                          resource.get_source(CONTEXT, REQUEST))


class TestDynamicSCSSResource(TestCase):

    def test_implements_interface(self):
        resource = DynamicSCSSResource('foo')
        verifyObject(ISCSSResource, resource)
        verifyObject(IDynamicSCSSResource, resource)

    def test_get_cachekey_must_be_implemented(self):
        resource = DynamicSCSSResource('foo')
        with self.assertRaises(NotImplementedError):
            resource.get_cachekey(None, None)

    def test_get_cachekey_can_be_passed_on_initialization(self):
        resource = DynamicSCSSResource('foo', cachekey='bar')
        self.assertEquals('bar', resource.get_cachekey(None, None))
